﻿/*
 * Created by SharpDevelop.
 * User: pshai
 * Date: 6.7.2016
 * Time: 10.24
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TradeServer.ScriptingDriver.DataObjects;
using TradeServer.ScriptingDriver.Interfaces;
using TicTacTec.TA.Library;

namespace Pshais.Indicators
{
	
	public class MovingAverageEnvelope : IIndicatorScript
	{
		int main_envLen = 40;
		double main_envShiftPrc = 2;
		Core.MAType main_envMaType = Core.MAType.Sma;
					
		double ma_mid, ma_top, ma_bot;
		
		#region IIndicatorScript implementation
		public void Init()
		{
			
		}
		public IndicatorResult GetResult(IndicatorContext context)
		{			
			try
            {
                IPriceInstrument instrument = context.PriceInstrument; // Its pre-created for speed based on the "TicksBack" property field

                // Lets do the calculation
                int position = instrument.Close.Count() - (1 + context.Offset); // minus 1 because of counting from 0 + account for the backtest
                
                
                
                int envBegin, envLen;
                var envOut = new double[position];
                var envRet = Core.MovingAverage(1, position, instrument.Close, main_envLen, main_envMaType, out envBegin, out envLen, envOut);
                
                if(envRet == Core.RetCode.Success)
                {
                	ma_mid = envOut[envLen-1];
                	ma_top = envOut[envLen-1] * (1 + main_envShiftPrc / 100);
                	ma_bot = envOut[envLen-1] * (1 - main_envShiftPrc / 100);
                	
                	// Open and Close prices
                	var op = instrument.Open[position];
                	var cp = instrument.Close[position];
                	
                	// Signaling rules
                	if(op < ma_bot || cp < ma_bot)
                		return IndicatorResult.BuyLong;
                	
                	if(op > ma_top || cp > ma_top)
                		return IndicatorResult.SellShort;
                }

            }
            catch (Exception ex) // This try catch block is an highly recommend practice
            {
                context.Logger.LogToFile(string.Format("Exception detected: {0}", ex));
            }

            return IndicatorResult.StayNoPosition; // Default output
		}
		
		public List<ScriptParameter> GetParameters()
		{
            var rtn = new List<ScriptParameter>
            {
                new ScriptParameter("MA Length", ScriptParameterType.Integer, main_envLen.ToString()),
                new ScriptParameter("MA Type", ScriptParameterType.String, main_envMaType.ToString()),
                new ScriptParameter("Shift %", ScriptParameterType.Decimal, main_envShiftPrc.ToString())
            };

            return rtn;
		}
		public void SetParameters(Dictionary<string, object> parameters)
		{
			main_envLen = ParseInt(parameters["MA Length"], main_envLen);
			main_envMaType = ParseMAType(parameters["MA Type"]);
			main_envShiftPrc = ParseDouble(parameters["Shift %"], main_envShiftPrc);
		}
		public List<ChartRecord> GetChartLines()
		{
            List<ChartRecord> chartLines = new List<ChartRecord>
            {
                new ChartRecord() { ChartIndex = 0, Name = "MA (Mid)", HexLineColor = "#FFFFFF" },
                new ChartRecord() { ChartIndex = 0, Name = "MA (Top)", HexLineColor = "#FF0000" },
                new ChartRecord() { ChartIndex = 0, Name = "MA (Bot)", HexLineColor = "#00FF00" }
            };

            return chartLines;
		}
		public List<decimal> GetChartData()
		{
            var chartData = new List<decimal>
            {
                (decimal)ma_mid,
                (decimal)ma_top,
                (decimal)ma_bot
            };


            return chartData;
		}
		public string Name {
			get {
				return "pshai's MA Envelope";
			}
		}
		public int TicksBack {
			get {
				var lb1 = Core.MovingAverageLookback(main_envLen, main_envMaType);
				return lb1 + 1;
			}
		}
		#endregion
		
		public static Core.MAType ParseMAType(object value)
		{
			Core.MAType val;
			switch( value.ToString().ToLower() )
			{
				case "sma":
					val = Core.MAType.Sma;
					break;
               	case "ema":
					val = Core.MAType.Ema;
					break;
               	case "dema":
					val = Core.MAType.Dema;
					break;
               	case "tema":
					val = Core.MAType.Tema;
					break;
               	case "trima":
					val = Core.MAType.Trima;
					break;
               	case "t3":
					val = Core.MAType.T3;
					break;
               	case "mama":
					val = Core.MAType.Mama;
					break;
               	case "kama":
					val = Core.MAType.Kama;
					break;
				case "wma":
					val = Core.MAType.Wma;
					break;
				default:
					val = Core.MAType.Sma;
					break;
			}
			
			return val;
		}
        
        int ParseInt(object value, int lastValue)
	    {
	    	try {
	    		return Convert.ToInt32(value.ToString());
	    	}
	    	catch
	    	{
	    		return lastValue;
	    	}
	    }
	    
	    double ParseDouble(object value, double lastValue)
	    {
	    	var numStyle = System.Globalization.NumberStyles.Any;
			var cult = System.Globalization.CultureInfo.InvariantCulture;
			
			try {
				double output;
				double.TryParse(
					value.ToString().Replace(',', '.'),
					numStyle,
					cult,
					out output);
				return output;
	    	}
	    	catch
	    	{
	    		return lastValue;
	    	}
	    }
	    
	    string ParseString(object value, string lastValue)
	    {
	    	try {
	    		return Convert.ToString(value);
	    	}
	    	catch
	    	{
	    		return lastValue;
	    	}
	    }
	}
}